use std::str::FromStr;

use anyhow::anyhow;
use serde::{Deserialize, de::DeserializeOwned};


#[derive(Deserialize)]
pub struct MemoryStorageConfig {
    pub max_size: usize,
    pub ttl: usize
}

#[derive(Deserialize)]
pub struct DiskStorageConfig {
    pub path: String,
    pub max_size: usize
}

#[derive(Deserialize)]
pub enum StorageStrategyConfig {
    Memory(MemoryStorageConfig),
    Disk(DiskStorageConfig)
}

#[derive(Deserialize)]
pub struct StorageConfig {
    pub strategy: StorageStrategyConfig,

    #[serde(with = "serde_regex")]
    pub regex: regex::Regex
}

#[derive(Deserialize, Default)]
pub struct YAMLConfig {
    pub storages: Vec<StorageConfig>,
    pub secret_key: Option<String>
}

pub async fn load() -> YAMLConfig {
    tokio::fs::read("config.yml")
        .await
        .map_err(|err| anyhow!(err))
        .and_then(|content| serde_yaml::from_slice::<YAMLConfig>(&content).map_err(|err| anyhow!(err)))
        .unwrap_or_default()
}