use anyhow::anyhow;
use axum::{
    http::StatusCode,
    routing::get, Router, extract::{Path, State}, response::IntoResponse,
};

use magic_crypt::{new_magic_crypt, MagicCryptTrait};
use tokio_stream::{self as stream, StreamExt};
use tokio_util::{bytes::Bytes};
use tracing::{warn};
use clap::Parser;

use std::{sync::Arc, path::PathBuf};

use crate::storages::StoragePool;

mod config;
mod storages;
mod routes;
mod state;

struct AppState {
    secret_key: String,
    authorized_commands: Vec<String>
}

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    tracing_subscriber::fmt::init();

    let config = config::load().await;
    let secret_key = config.secret_key.clone();
    let storage_pool = StoragePool::from_config(config.storages);

    let app_state = Arc::new(state::AppState::new(storage_pool, secret_key));

    println!("{}", config.secret_key.clone().unwrap_or_default());

    let app = Router::new()
        .route("/unsecure/*src", get(routes::handle_unsecure))
        .route("/secure/*data", get(routes::handle_secure))
        .with_state(app_state);

    let listener = tokio::net::TcpListener::bind("0.0.0.0:3000").await?;
    axum::serve(listener, app).await?;

    Ok(())
}

/*async fn handle_secure(Path(data): Path<String>, State(state): State<Arc<AppState>>) -> Result<impl IntoResponse, MyAnyhow> {
    let mc = new_magic_crypt!(&state.secret_key, 256);
    let decrypt = mc.decrypt_base64_to_string(data)?;
    let split = decrypt.splitn(2, "/").map(String::from).collect::<Vec<String>>();

    let opts = split.get(0).ok_or_else(|| anyhow::anyhow!("opts not found"))?;
    let src = split.get(1).ok_or_else(|| anyhow::anyhow!("src not found"))?;

    let bytes = handle(opts.clone(), src.clone(), state).await?;
    Ok(axum::body::Body::from(bytes))
}

async fn handle_unsecure(Path((opts, src)): Path<(String, String)>, State(state): State<Arc<AppState>>) -> Result<impl IntoResponse, MyAnyhow> {
    let bytes = handle(opts, src, state).await?;

    Ok(axum::body::Body::from(bytes))
}*/

/*async fn handle(_opts: String, src: String) -> anyhow::Result<impl Stream<Item = Result<Bytes, reqwest::Error>>> {
    let client = reqwest::Client::new();
    let res = client.get(src).send().await?;
    let stream = res.bytes_stream();

    Ok(stream)
}*/

/*async fn handle(opts: String, src: String, app_state: Arc<AppState>) -> anyhow::Result<Bytes, MyAnyhow> {
    let qs_opts = queryst::parse(&opts).map_err(|err| anyhow::anyhow!("{}", err.message))?;
    
    let client = reqwest::Client::new();
    let res = client.get(src.clone()).send().await?;
    let bytes = res.bytes().await?;

    if let Some(storage) = &app_state.storage {

    }


    Ok(bytes)
}*/

