use std::{sync::Arc};

use axum::{extract::{Path, State}, response::IntoResponse, http::StatusCode};
use magic_crypt::{new_magic_crypt, MagicCryptTrait};
use tokio_stream::Stream;
use tokio_util::bytes::Bytes;

use crate::state::AppState;


pub async fn handle_secure(Path(data): Path<String>, State(state): State<Arc<AppState>>) -> Result<impl IntoResponse, MyAnyhow> {
    let mc = new_magic_crypt!(&state.secret_key.clone().unwrap(), 256);
    let decrypt = mc.decrypt_base64_to_string(data)?;
    let split = decrypt.splitn(2, "/").map(String::from).collect::<Vec<String>>();

    let opts = split.get(0).ok_or_else(|| anyhow::anyhow!("opts not found"))?;
    let src = split.get(1).ok_or_else(|| anyhow::anyhow!("src not found"))?;

    let bytes = handle(opts.clone(), src.clone()).await?;
    Ok(axum::body::Body::from_stream(bytes))
}

pub async fn handle_unsecure(Path((opts, src)): Path<(String, String)>, State(state): State<Arc<AppState>>) -> Result<impl IntoResponse, MyAnyhow> {
    let bytes = handle(opts, src).await?;

    Ok(axum::body::Body::from_stream(bytes))
}

async fn handle(_opts: String, src: String) -> anyhow::Result<impl Stream<Item = Result<Bytes, reqwest::Error>>> {
    let client = reqwest::Client::new();
    let res = client.get(src).send().await?;
    let stream = res.bytes_stream();

    Ok(stream)
}

pub struct MyAnyhow(anyhow::Error);

impl IntoResponse for MyAnyhow {
    fn into_response(self) -> axum::response::Response {
        (StatusCode::INTERNAL_SERVER_ERROR, self.0.to_string()).into_response()
    }
}

impl <T>From<T> for MyAnyhow where T: Into<anyhow::Error> {
    fn from(value: T) -> Self {
        Self(value.into())
    }
}