pub mod memory;
pub mod disk;

use async_trait::async_trait;

use crate::config::StorageConfig;

use self::{memory::MemoryStorage, disk::DiskStorage};

#[async_trait]
pub trait Storage {}

pub struct StoragePool {
    storages: Vec<Box<dyn Storage + Sync + Send>>
}

impl StoragePool {
    pub fn from_config(items: Vec<StorageConfig>) -> Self {
        Self { storages: items.into_iter().map(create_storage).collect() }
    }
}

fn create_storage(item: StorageConfig) -> Box<dyn Storage + Sync + Send> {
    match item.strategy {
        crate::config::StorageMethodConfig::Memory(config) => Box::new(MemoryStorage::new(config)),
        crate::config::StorageMethodConfig::Disk(config) => Box::new(DiskStorage::new(config)),
    }
}